<?php
namespace Teknisa\EST\Modules\Relatorios\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Teknisa\EST\Modules\Relatorios\Services\Ger09700ProjecaoDeEstoqueService;
use Teknisa\EST\Controllers\TeknisaCrud;

class Ger09700ProjecaoDeEstoqueController extends TeknisaCrud {
    
    protected $crudService;
	protected $dataSourceManager;
	protected $parameterBag;
	protected $environment;
    protected $gerService;
    protected $dataSourceName;
    // protected $parameterBagColumns = array("P_CDFILESTPROJ", "P_NRSEQESTPROJ", "P_DTPERLEVINI", "P_DTPERLEVFIN");
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, Ger09700ProjecaoDeEstoqueService $gerService) {
        parent::__construct($crudService,$parameterBag);
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->gerService = $gerService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        $reportData = array_filter($conditions,function($condition){
            return $condition['columnName'] == 'REPORT_NAME';
        });
        
        $reportName = $reportData[0]['value'];
        $parameters = $this->parametersBuilder($conditions,$reportName);
        switch($reportName){
            case 'GER09700':{
                $parameters['P_NRSEQESTPROJ'] = $this->gerService->executaProcedureProjecaoDeEstoque(
                    $parameters['P_FILIPROC'],
                    $parameters['P_CDFILIAL'],
                    $parameters['P_DTLEVINI'],
                    $parameters['P_DTLEVFIN'],
                    $parameters['P_PRODINI'],
                    $parameters['P_PRODFIN'],
                    $parameters['P_FILESTOQ'],
                    $parameters['P_IDGERSOL'],
                    $parameters['P_CONSSOL'],
                    $parameters['P_CONSAF'],
                    $parameters['P_CONSEST'],
                    $parameters['P_CONSROM'],
                    $parameters['P_FILIROM'],
                    $parameters['P_SERIROM'],
                    $parameters['P_OPERADOR'],
                    $parameters['P_TODOSPRODEST']
                );
                
                $this->dataSourceName = "projeta_estoque";
                
                $newFilterCriteria = new FilterCriteria($this->dataSourceName, []);
                $newFilterCriteria->addCondition('P_CDFILESTPROJ',$parameters['P_FILIPROC']);
                $newFilterCriteria->addCondition('P_NRSEQESTPROJ',$parameters['P_NRSEQESTPROJ']);
                $newFilterCriteria->addCondition('P_DTPERLEVINI',$parameters['P_DTLEVINI']);
                $newFilterCriteria->addCondition('P_DTPERLEVFIN',$parameters['P_DTLEVFIN']);
                
                $newRequest = new Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::find($newRequest, $response);
                
                $this->gerService->executaDeleteProjecaoDeEstoque($parameters['P_FILIPROC'], $parameters['P_NRSEQESTPROJ']);
                break;
            }
            case 'GER09700Dis':{
                $parameters['P_NRSEQESTPROJ'] = $this->gerService->executaProcedureProjecaoDeEstoqueDia($parameters);
                
                $this->dataSourceName = "projeta_estoque_dia";
                
                $newFilterCriteria = new FilterCriteria($this->dataSourceName, []);
                $newFilterCriteria->addCondition('P_CDFILESTPROJ',$parameters['P_CDFILIAL']);
                $newFilterCriteria->addCondition('P_DTPERLEVINI',$parameters['P_DTLEVINI']);
                $newFilterCriteria->addCondition('P_DTPERLEVFIN',$parameters['P_DTLEVFIN']);
                $newFilterCriteria->addCondition('P_NRTIPOCUSTO',$parameters['P_NRTIPOCUSTO']);
                $newFilterCriteria->addCondition('P_NRSEQESTPROJ',$parameters['P_NRSEQESTPROJ']);
                
                $newRequest = new Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $newResponse = new Response();
                
                parent::find($newRequest, $newResponse);
                $rows = $newResponse->getDataSets()[0]->getRows();
                
                if($rows){
                    foreach($rows as &$row){
                        $row['POSICAO'] = $this->gerService->getStockInfo($row) ?? 0;
                        $row['QTDE'] = $this->gerService->getReserveInfo($row) ?? 0;
                        $row['SALDO'] = $row['POSICAO'] + $row['ENTRADAS'] - $row['SAIDAS'];
                        $row['VRTOTAL'] = $row['VRCUSTOPROD'] * $row['POSICAO'];
                        $row['UNIDADEAGRUP'] = $row['CDFILESTPROJ'] . ' - ' . $row['NMFILIAL'];
                    }
                }
                $this->gerService->executaDeleteProjecaoDeEstoque($parameters['P_FILIPROC'], $parameters['P_NRSEQESTPROJ']);
                $response->addDataSet(new DataSet("projeta_estoque", $rows));
                break;
            }
        }
    }
    
    //this function builds parameter bag
    public function parametersBuilder($conditions,$reportName){
        $parameters = [];
        
        // Set defult params
        $parameters['P_IDGERSOL'] = null;
        $parameters['P_FILIROM'] = null;
        $parameters['P_SERIROM'] = null;
        $parameters['P_FILESTOQ'] = null;
        $parameters['P_NRSEQESTPROJ'] = null;
        $parameters['P_ENTRINI'] = null;
        $parameters['P_ENTRFIN'] = null;
        $parameters['P_SAIDAINI'] = null;
        $parameters['P_SAIDAFIN'] = null;
        $parameters['P_DTLEVINI'] = null;
        $parameters['P_DTLEVFIN'] = null;
        
        
        foreach($conditions as $condition) {
            switch($condition["columnName"]){
                case 'CDFILIAL':
                    $parameters['P_FILIPROC'] = $this->environment->getCdFilial();
                    $parameters['P_CDFILIAL'] = $condition["value"];
                    break;
                case 'DTLEV':
                    $parameters['P_DTLEVINI'] = $condition["value"][0];
                    $parameters['P_DTLEVFIN'] = $condition["value"][1];
                    break;
                case 'DTENTRPEN':
                    $parameters['P_ENTRINI'] = $condition["value"][0];
                    $parameters['P_ENTRFIN'] = $condition["value"][1];
                    break;
                case 'DTSAIPEN':
                    $parameters['P_SAIDAINI'] = $condition["value"][0];
                    $parameters['P_SAIDAFIN'] = $condition["value"][1];
                    break;
                case 'CDPRODINI':
                    $parameters['P_PRODINI'] = $condition["value"];
                    break;
                case 'CDPRODFIN':
                    $prodFin = $condition["value"] . "ZZZZZZZZZZZZZZZZZZZZ";
                    $parameters['P_PRODFIN'] = substr($prodFin, 0, 20);
                    break;
                case 'TFILESTOQ':
                    if($condition["value"] == 'T') 
                        $parameters['P_FILESTOQ'] = null;
                    break;
                case 'NMFILESTOQ':
                    if(is_array($condition["value"])) {
                        if(count($condition["value"]) != 0) {
                            $filialstr = '_' . $this->gerService->formatMutipleFields($condition["value"]);
                            $parameters['P_FILESTOQ'] = $filialstr;
                        } else 
                            $parameters['P_FILESTOQ'] = null;
                    } else 
                        $parameters['P_FILESTOQ'] = null;
                    break;
                case 'GERSOL':
                    $parameters['P_IDGERSOL'] = $condition["value"];
                    break;
                case 'CONSSOL':
                    $parameters['P_CONSSOL'] = $condition["value"];
                    break;
                case 'CONSAF':
                    $parameters['P_CONSAF'] = $condition["value"];
                    break;
                case 'CONSEST':
                    $parameters['P_CONSEST'] = $condition["value"];
                    break;
                case 'CONSROM':
                    $parameters['P_CONSROM'] = $condition["value"];
                    break;
                case 'TFILIROM':
                    if($condition["value"] == 'T') 
                        $parameters['P_FILIROM'] = null;
                    break;
                case 'NMFILIROM':
                    if(is_array($condition["value"])) {
                        if(count($condition["value"]) != 0) {
                            $filialstr = '_' . $this->gerService->formatMutipleFields($condition["value"]);
                            $parameters['P_FILIROM'] = $filialstr;
                        } else 
                            $parameters['P_FILIROM'] = null;
                    } else 
                        $parameters['P_FILIROM'] = null;
                    break;
                case 'TSERIROM':
                    if($condition["value"] == 'T') 
                        $parameters['P_SERIROM'] = 'T';
                    break;
                case 'DSSERIE':
                     if(is_array($condition["value"])) {
                        if(count($condition["value"]) != 0) {
                            $seriestr = '_' . $this->gerService->formatMutipleFields($condition["value"]);;
                            $parameters['P_SERIROM'] = $seriestr;
                        } else 
                            $parameters['P_SERIROM'] = null;
                    } else 
                        $parameters['P_SERIROM'] = null;
                    break;
                case 'CDOPERADOR':
                    $parameters['P_OPERADOR'] = $condition["value"];
                    break;
                case 'CDTIPOCUSTO':
                    $parameters['P_NRTIPOCUSTO'] = $condition["value"];
                    break;
                case 'TODOSPRODEST':
                    $parameters['P_TODOSPRODEST'] = $condition["value"];
                    break;
                default:
            }
        }
        return $parameters;
    }
}