<?php
/* PercorreGRRF.
 * @package Logic
 * @author francismararaujo
 */
class PercorreGRRF{

    private $_parametros;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_mesCompetencia;
    private $_nrTipoModalidadeCalculo;
    private $_tipoMovimento;
    private $_itemGrupo;
    private $_vinculom;
    private $_tipoHierarquia;
    private $_hierarquiaOrg;
    private $_calculoFolha;
    private $_grrfMagnetico;
    private $_registro00;
    private $_registro90;
    private $_pessoa;
    private $_estruturacao;
    private $_formulario;
    private $_nrRecolhimento;
    private $_idModalidade;
    private $_nomeInicial;
    private $_nomeFinal;
    private $_dtInicial;
    private $_dtFinal;
    private $_dtRecolhGRRF;
    private $_dsOrdem;
    private $_nrTpHierarquiaOrg;
    private $_tpInscHierarq;
    private $_nrTpHierarqTomador;
    private $_tpInscHierarqTomador;
    private $_idProcessaVinculo;
    private $_idGeraMeioMagnetico;
    private $_filePath;
    private $_nrEstruturaResp;
    private $_tpInscResp;
    private $_nrPessoaResp;
    private $_idRecolhimentoFGTS;
    private $_dtFGTS;
    private $_idRecolhimentoGPS;
    private $_dtGPS;
    private $_nrGrpIncide;
    private $_listaNrCalculoFolha;
    private $_retrocesso;
    private $_competencia13;

    private $_grrf;
    private $_listaTipoRegistro;
    private $_message;

    /* Construtor.
     * @param string $tpProcessamento Tipo de processamento (MENSAL / COMPETENCIA 13).
     * @param integer $nrFormulario Numero do formulario.
     * @author francismararaujo
     */

    public function __construct($tpProcessamento, $nrFormulario){

        $this->_parametros = new Parametros(Ambiente::getNrorg(), Ambiente::getCdOperador());
        $this->_nrOrg = $this->_parametros->getNrOrgTrabalho();
        $this->_nrOrgPadrao = $this->_parametros->getNrOrgPadrao();
        $this->_nrEstruturaTrab = \FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = new TkDateTime($this->_parametros->getMesCompetencia());
        $this->_nrTipoModalidadeCalculo = $this->_parametros->getTipoModalidadeCalculo();
        $this->_tipoMovimento = FpatipomovimentEntity::TP_MOV_MENSAL;
        $this->_itemGrupo = new FpaGrpitemEntity();
        $this->_vinculom = new GpevinculomEntity();
        $this->_tipoHierarquia = new TipohierarqorgEntity();
        $this->_hierarquiaOrg = new HierarquiaorgEntity();
        $this->_calculoFolha = new FpacalculofolhaEntity();
        $this->_grrfMagnetico = new FpagrrfmagnetEntity();
        $this->_registro00 = new Registro00GRRF();
        $this->_registro90 = new Registro90GRRF();
        $this->_pessoa = new GpepessoaEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_estruturacao = new Estruturacao($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_inicializaVariaveis($nrFormulario);

        switch (strtoupper($tpProcessamento)){

            case Parametros::PROC_COMPETENCIA_MENSAL :
                $this->_retrocesso = false;
                $this->_competencia13 = false;
                break;
            case Parametros::PROC_COMPETENCIA_13 :
                $this->_retrocesso = false;
                $this->_competencia13 = true;
                break;
            case Parametros::PROC_RETROCESSO :
                $this->_retrocesso = true;
                $this->_competencia13 = false;
                break;
            default:
                tk_message('Tipo de processo '.$tpProcessamento.' invalido', 'warning');

        }

        //objeto grrf logic
        $this->_grrf  = new GRRF(
            $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_grrfMagnetico, $this->_formulario,
            $this->_competencia13, $this->_itemGrupo, $this->_listaNrCalculoFolha
        );

    }

    /* Inicializa as variaveis do formulario.
     * @param integer $nrFormulario Numero do formulario.
     * @author francismararaujo
     */

    public function _inicializaVariaveis($nrFormulario) {
        $this->_buscaDadosFormulario($nrFormulario);
        $this->_itemGrupo->carregaItensGrupoPorGrupoIncidencia($this->_nrOrg, $this->_nrOrgPadrao, $this->_nrGrpIncide);
        // Cria lista de nr_calculo_folha.
        $this->_listaNrCalculoFolha = $this->_calculoFolha->carregaListaNrCalculoFolhaTodasOcorrencias(
            $this->_nrOrg, $this->_tipoMovimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo
        );
    }

    /* Busca dados do formulario.
     * @param integer $nrFormulario Numero do formulario.
     * @return void
     * @author francismararaujo
     */

    private function _buscaDadosFormulario($nrFormulario){
        $formulario = new FpaformgrrfEntity();
        $this->_formulario = $formulario->retornaDadosFormularioGRRF($this->_nrOrg, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_nrRecolhimento = $this->_formulario->getNrrecolhimento();
            $this->_idModalidade = $this->_formulario->getIdmodalidade();
            $this->_nomeInicial = $this->_formulario->getNomeinicial();
            $this->_nomeFinal = $this->_formulario->getNomefinal();
            $this->_dtInicial = $this->_mesCompetencia;
            $this->_dtFinal = FPA::getUltimoDiaMes();
            $this->_dtRecolhGRRF = $this->_formulario->getDtRecolhGRRF();
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_nrTpHierarquiaOrg = $this->_formulario->getNrhierarqorg();
            $this->_tpInscHierarq = $this->_formulario->getIdtpinschierarq();
            $this->_nrTpHierarqTomador = $this->_formulario->getNrhierartomador();
            $this->_tpInscHierarqTomador = $this->_formulario->getIdtpinschiertom();
            $this->_idProcessaVinculo = $this->_formulario->getIdprocvinculo();
            $this->_idGeraMeioMagnetico = $this->_formulario->getIdgerameiomag();
            $this->_filePath = $this->_formulario->getDscaminhoarquivo();
            $this->_nrEstruturaResp = $this->_formulario->getNrestrutrespons();
            $this->_tpInscResp = $this->_formulario->getIdtpinscrespons();
            $this->_nrPessoaResp = $this->_formulario->getNrpessoarespons();
            $this->_idRecolhimentoFGTS = $this->_formulario->getIdrecolhfgts();
            $this->_dtFGTS = $this->_formulario->getDtrecolhfgts();
            $this->_idRecolhimentoGPS = $this->_formulario->getIdrecolhgps();
            $this->_dtGPS = $this->_formulario->getDtrecolhgps();
            $this->_nrGrpIncide = $this->_formulario->getNrgrpincide();
        }
    }

    /* Percorre os vinculos para geracao dos registros.
     * @throws Exception Propaga excecao.
     * @return void
     * @author francismararaujo
     */

    public function percorreVinculos(){

        //se nao for retrocesso
        if ($this->_retrocesso === false) {

            //inicializa lista do magnetico.
            $this->_grrfMagnetico->inicializaListaRegistros(
                $this->_mesCompetencia, RegistroGRRF::LISTA_TIPO_REG_EMP, $this->_nrOrg
            );

            //se "processa vinculo" estiver marcado no formulario.
            if ($this->_idProcessaVinculo === true) {

                //retorna todos os vinculos demitidos.
                $vinculos = $this->_vinculom->retornaVinculosDemitidosMesCompetencia(
                    $this->_nrOrg, $this->_mesCompetencia, $this->_nomeInicial, $this->_nomeFinal, $this->_dtInicial,
                    $this->_dtFinal, $this->_dsOrdem, $this->_nrEstruturaTrab
                );

                //se existir os vinculos
                if(!empty($vinculos)){
                    //percorre os vinculos
                    foreach ($vinculos as $vinculo) {
                        //somente avalia os funcionarios com vinculo empregaticio.
                        if ($vinculo->getNrvinculoempreg() !== false) {
                            try{
                                $this->_grrf->gerarGRRF($vinculo, $this->_tpInscHierarq, $this->_tpInscHierarqTomador);
                                tk_message('Arquivo gerado', 'success');
                            }catch (Exception $e){
                                tk_message($e->getMessage(), 'warning');
                            }
                        }else{
                            tk_message('Vínculo empregatício não parametrizado.', 'warning');
                        }

                        \Setforms2\Runtime\Connection::getInstance()->clear();
                    }
                }else{
                    tk_message('Nenhum vínculo demitido foi encontrado para o mês de competência.', 'warning');
                }

                // Modifica valores nas strings ja montadas.
                //$this->_grrf->modificaSalarioMaternidadeFamilia();

                // criar metodo no GRRF para fazer a modificacao

                // So persiste se a lista nao estiver vazia.
                $qtdRegistros = $this->_grrfMagnetico->countItems();

                //$qtdRegistros = count($this->_grrfMagnetico->getItems());
                if ($qtdRegistros > 0) {

                    $this->_grrfMagnetico->persist();

                }

            }

            //se "gerar magnetico" estiver marcado no formulario.
            if ($this->_idGeraMeioMagnetico === true) {

                $this->_geraMagnetico();

            }

        }else{

            //se escolher retrocesso
            $this->_grrf->retroceder();

        }

    }

    /* Enter gerar magnetico
     * @return void
     * @author francismararaujo
     */

    private function _geraMagnetico(){

        //Chama método para gerar o arquivo, passando os parâmetros: nome da pasta, nome do arquivo e extensão arquivo
        $nomePasta = "GRRF";
        $nomeArquivo = "GRRF.RE";
        $extensao = ".RE";
        $arquivo = \FPA::geraArquivo($nomePasta, $nomeArquivo, $extensao);

        //prepara a primeira linha do arquivo
        $this->_preparaRegistro00();

        //informacoes sobre o responsavel do arquivo.
        $this->_registro00->geraRegistro00();

        //armazenar o registro 00 com quebra de linha.
        $arquivo->add($this->_registro00->getLinhaRegistro().RegistroGRRF::QUEBRA_LINHA_CR_LF);
        //$arquivo.=$this->_registro00->getLinhaRegistro().RegistroGRRF::QUEBRA_LINHA_CR_LF;

        //busca da base os resgistros daquela competencia, ordenados por inscricao e tipo de registro.
        $resultado = $this->_grrfMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg);

        foreach ($resultado as $item) {

            //percorre o resultado armazenando os registros com quebra de linha.
            $arquivo->add($item->getDslinharegistro().RegistroGRRF::QUEBRA_LINHA_CR_LF);
            //$arquivo.= $item->getDslinharegistro().RegistroGRRF::QUEBRA_LINHA_CR_LF;

        }

        //cria a ultima linha do registro,
        $this->_registro90->geraRegistro90();

        //adiciona
        $arquivo->add($this->_registro90->getLinhaRegistro());

        $arquivo->save(false, null, 0777);

        //$arquivo->download("'http://folha.teknisa.com/".\FPA::getNrorgTrabalho()."/GRRF/GRRF.RE");
        $arquivo->download(substr($_SERVER['REQUEST_URI'], 0, strlen($_SERVER['REQUEST_URI']) - 12)."/".\FPA::getNrorgTrabalho()."/".$nomePasta."/".$nomeArquivo);
    }


    private function _preparaRegistro00(){

        $estrutura = $this->_estruturacao->retornaEstrutura($this->_nrEstruturaResp);

        //valida se a estrutura e um objeto
        if (is_object($estrutura) === true){


            $estrutura->endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
            $pessoa = $this->_pessoa->retornaPessoaAtual($this->_nrPessoaResp);

            if ($this->_tpInscResp === RegistroGRRF::CNPJ) {
                $this->_registro00->setTpInscricao(RegistroGRRF::CNPJ);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdcnpjestrut());
            } elseif ($this->_tpInscResp === RegistroGRRF::CEI) {
                $this->_registro00->setTpInscricao(RegistroGRRF::CEI);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdceiestrut());
            }

            $this->_registro00->setPessoaResp($pessoa);
            $this->_registro00->setEstruturaResp($estrutura);
            $this->_registro00->setCompetencia($this->_mesCompetencia);
            $this->_registro00->setCodigoRecolhimento($this->_nrRecolhimento);
            $this->_registro00->setModalidade($this->_idModalidade);
            $this->_registro00->setIdRecolhimentoFGTS($this->_idRecolhimentoFGTS);
            $this->_registro00->setDtFGTS($this->_dtFGTS);
            $this->_registro00->setDtRecolhGRRF($this->_dtRecolhGRRF);
            $this->_registro00->setIdRecolhimentoGPS($this->_idRecolhimentoGPS);
            $this->_registro00->setDtGPS($this->_dtGPS);
            $this->_registro00->setCompetencia13($this->_competencia13);

        }else{

            tk_message('Estrutura não encontrada.', 'warning');

        }


    }

}